/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservationReplaceBuilder.java,v 1e006e4ad142 2010/05/15 14:02:51 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.simple;

import javax.annotation.Nonnull;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Set;
import it.tidalwave.util.Initializer;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.Observation.ReplaceBuilder;
import it.tidalwave.bluebill.observation.Observer;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SimpleObservationReplaceBuilder.java,v 1e006e4ad142 2010/05/15 14:02:51 fabrizio $
 *
 ******************************************************************************/
public class SimpleObservationReplaceBuilder extends SimpleObservationBaseBuilder implements Observation.Builder, Observation.ReplaceBuilder
  {
    @Nonnull
    private final Observation originalObservation;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected SimpleObservationReplaceBuilder (final @Nonnull SimpleObservationSet observationSet,
                                               final @Nonnull Observation originalObservation)
      {
        super(observationSet);
        this.originalObservation = originalObservation;
        date(originalObservation.getDate());
        location(originalObservation.getLocation());
//        note(originalObservation.getNote());

        for (final Observer observer : originalObservation.findObservers().results())
          {
            observer(observer);
          }

        for (final ObservationItem observationItem : originalObservation.findObservationItems().results())
          {
            item(observationItem.getObservable(), observationItem.getCardinality());
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Observation apply()
      {
        if (date == null)
          {
            return null;
          }

        final Set<ObservationItem> observationItems = new HashSet<ObservationItem>();
        final SimpleObservation observation = new SimpleObservation(observationSet, date, location, observers, observationItems, note, origin);

        for (final ObservableWithCardinality observableWithCardinality : observablesWithCardinality)
          {
            ObservationItem observationItem = new SimpleObservationItem(observation,
                                                                        observableWithCardinality.observable,
                                                                        observableWithCardinality.cardinality);
            observationItems.add(observationItem);
            final Initializer<ObservationItem> initializer = initializerMap.get(observableWithCardinality);

            if (initializer != null)
              {
                initializer.initialize(observationItem);
              }
          }

        observationSet.replaceObservation(originalObservation, observation);
        return observation;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ReplaceBuilder removeItem (final @Nonnull ObservationItem observationItem)
      {
        for (final Iterator<ObservableWithCardinality> i = observablesWithCardinality.iterator(); i.hasNext(); )
          {
            final ObservableWithCardinality observableWithCardinality = i.next();

            // TODO: is it enough to compare the cardinality? What about extra attributes?
            if (observableWithCardinality.observable.equals(observationItem.getObservable()) &&
                observableWithCardinality.cardinality.equals(observationItem.getCardinality()))
              {
                i.remove();
                initializerMap.remove(observableWithCardinality);
                break;
              }
          }

        return this;
      }
  }
