/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservationSet.java,v daa6614e635a 2010/04/29 12:24:58 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.simple;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import it.tidalwave.util.Id;
import it.tidalwave.util.Initializer;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.observation.Finder;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.Observation.Builder;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.Source;
import it.tidalwave.bluebill.observation.spi.ObservationSetSupport;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class SimpleObservationSet extends ObservationSetSupport implements ObservationSet, Externalizable
  {
    private static final String CLASS = SimpleObservationSet.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    private static final long serialVersionUID = 636535648437593487L;

    private List<Observation> observations = new ArrayList<Observation>();

    private final AsSupport asSupport = new AsSupport();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public <T> T findOrCreate (Class<T> clazz, String displayName, Initializer<T> initializer)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public <T> T findById (Class<T> clazz, Id id)
      throws NotFoundException
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> Finder<T> find (final @Nonnull Class<T> clazz)
      {
        if (clazz.equals(Observation.class))
          {
            return (Finder<T>)new SimpleFinderSupport<Observation>(observations);
          }

        throw new UnsupportedOperationException("Not supported yet.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Builder createObservation()
      {
        return new SimpleObservationBuilder(this);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Source createSource (String uri, String displayName)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clear()
      {
        observations.clear();
        fireNotifyCleared();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public final <T> T as (final @Nonnull Class<T> type, final @Nonnull NotFoundBehaviour<T> nfb)
      {
        return asSupport.as(type, nfb);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public final <T> T as (final @Nonnull Class<T> type)
      {
        return asSupport.as(type);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Lookup getLookup()
      {
        return asSupport.getLookup();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void addCapabilities (final @Nonnull Object ... capabilities)
      {
        asSupport.capabilities.addAll(Arrays.asList(capabilities));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void readExternal (final @Nonnull ObjectInput oi)
      throws IOException, ClassNotFoundException
      {
        super.readExternal(oi);
        observations = (List<Observation>)oi.readObject();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void writeExternal (final @Nonnull ObjectOutput oo)
      throws IOException
      {
        super.writeExternal(oo);
        oo.writeObject(observations);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void addObservation (final @Nonnull SimpleObservation observation)
      {
        observations.add(observation);
        fireObservationAdded(observation);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void replaceObservation (final @Nonnull Observation originalObservation,
                                       final @Nonnull SimpleObservation observation)
      {
        final int index = observations.indexOf(originalObservation);

        if (index < 0)
          {
            throw new IllegalStateException("The original observation has been deleted in the meantime " + originalObservation);
          }

        observations.set(index, observation);
        fireObservationChanged(observation, originalObservation);
      }
  }
